/* (c) Copyright 2001 Josh Levenberg */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <GL/glut.h>
#include "glutmain.h"
#include "world.h"
#include "init.h"
#include "glswmath.h"

static GlutState *sCurMode=0;

int gWidth=640;
int gHeight=480;
float gFov=45.0;
float gNear;
float gFar=(NUM_TRENCH_SEGMENTS+1)*TRENCH_DEPTH;

static bool sFullScreen=true;

/* window size to restore when switching back from full screen */
static int sWidthSaved=640;
static int sHeightSaved=480;

void DummyRenderFunction(void)
  {
  }

void SwitchToGlutState(GlutState *newstate)
  {
    if (newstate != sCurMode)
      {
        if (sCurMode)
          sCurMode->close();
        sCurMode=newstate;
        if (sCurMode)
          {
            glutDisplayFunc(sCurMode->draw);
            glutIdleFunc(sCurMode->idle);
            glutMouseFunc(sCurMode->mouseclick);
            glutMotionFunc(sCurMode->mousemove);
            glutPassiveMotionFunc(sCurMode->mousemove);

            sCurMode->init();
          }
        else
          {
            glutDisplayFunc(DummyRenderFunction);
            glutIdleFunc(0);
            glutMouseFunc(0);
            glutMotionFunc(0);
            glutPassiveMotionFunc(0);
          }
      }
  }

static GlutState *sLastMode=0;

/* Pause the game when minimizing by switching the state to 0,
   but save the state so we can switch back when the window is restored */
static void GlutVisible(int state)
  {
    /* called if visibility status of window changes */
    switch( state )
      {
        case GLUT_VISIBLE:
          if (sLastMode)
            SwitchToGlutState(sLastMode);
          break;
        case GLUT_NOT_VISIBLE:
          sLastMode = sCurMode;
          SwitchToGlutState(0);
          break;
        default:
          printf("Should not get to default case in GlutVisible()\n");
          exit(0);
      }
  }

static void GlutResize(int width, int height)
  {   
    gWidth=width;
    gHeight=height;
    glViewport(0, 0, width, height);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(gFov, (float)gWidth/(float)gHeight, gNear, gFar);
    glMatrixMode(GL_MODELVIEW);
  }

static void GlutKeyboard(unsigned char key, int x, int y)
  {
    if (key=='q' || key=='Q' || key==27 /* ESC */)
      {
        SwitchToGlutState(0);
        exit(0);
      }
    if (key=='p' || key=='P')
      { /* "Pause" -- actually a side effect of minimizing */
        glutIconifyWindow();
      }
  }

/* Needs to save window size when switching to full screen
   so we can restore the window when switching back.
   Note that according to the GLUT documentation, 
   glutReshapeWindow or glutPositionWindow will exit 
   full screen mode.

   If I was feeling adventurous, could use GlutGameMode
   to get higher performance.
 */
static void ToggleFullScreen()
  {
    if (sFullScreen)
      { /* Exit full screen mode */
        glutReshapeWindow(sWidthSaved, sHeightSaved);
        sFullScreen=false;
      }
    else
      { /* Enter full screen mode */
        sWidthSaved=gWidth;
        sHeightSaved=gHeight;
        glutFullScreen();
        sFullScreen=true;
      }
  }

static void GlutKeyboardSpecial(int key, int x, int y)
  {
    if (key==GLUT_KEY_F1 || key==GLUT_KEY_F4)
      ToggleFullScreen();
  }

bool CheckForGLError(const char *where)
  {
    GLenum err;
    err=glGetError();
    if (err!=GL_NO_ERROR)
      {
        const GLubyte *errString;
        errString = gluErrorString(err);
        fprintf (stderr, "OpenGL Error in %s: %s\n", where, errString);
        return true;
      }
    return false;
  }

/* Main entry point.  Do some basic GLUT window and callback setup,
   call Initialize, then start the main loop.
 */
int main(int argc, char *argv[])
  {
    /* should be over 2 or 8/3 instead of 4 but I put it a little 
       margin of safety in case the window is unusually wide */
    gNear=PLAYER_HEIGHT/(4.*tan(DegreesToRadians(gFov/2)));

    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_RGB | GLUT_DEPTH | GLUT_DOUBLE);
    glutInitWindowSize(gWidth, gHeight);
    /* glutInitWindowPosition(100, 100); */
    glutCreateWindow("GL SW by Josh Levenberg");
    glutReshapeFunc(GlutResize);
    glutVisibilityFunc(GlutVisible);
    glutKeyboardFunc(GlutKeyboard);
    glutSpecialFunc(GlutKeyboardSpecial);
    glutFullScreen();
 
    CheckForGLError("(GLUT window creation)");

    Initialize();
    
    CheckForGLError("Initialization");

    glutMainLoop();

    SwitchToGlutState(0);

    return 0;
  }
