/* (c) Copyright 2001 Josh Levenberg */

#include <assert.h>
#include <math.h>
#include <stdio.h>
#include <GL/glut.h>
#include "glutmain.h"
#include "world.h"
#include "model.h"
#include "glswmath.h"
#include "draw.h"
#include "text.h"

#define MESSAGE_SCALE (1.f/600.f)
#define SCORE_SCALE   (1.f/2400.f)

void DrawWorld(void)
  {
    int i;

    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    glLoadIdentity();

    gluLookAt(/* camera position: */
              gPlayerPos[0], gPlayerPos[1], gPlayerPos[2],
              /* look at vector: */
              gPlayerPos[0], gPlayerPos[1]+1.0f, gPlayerPos[2],
              /* up vector: */
              0.f, 0.f, 1.f); 

    /* ---- Draw opaque things first ---- */

    glDisable(GL_BLEND);

    /* Draw trench */
    glPushMatrix();
    for (i=0; i<NUM_TRENCH_SEGMENTS; ++i)
      {
        glCallList(gModelDisplayList+Model_Trench);
        glTranslatef(0.f, TRENCH_DEPTH, 0.f);
      }
    glPopMatrix();

    /* Draw shots */
    if (gShootCountdown)
      {
        static int guns[4][2] = { { -1, -1 }, { 1, 1 }, { -1, 1 }, { 1, -1 } } ;
        float shotsrc[3];
        float shotdiff[2];
        int whichgun = (int) (gShootCountdown*4/SHOOT_TIME);
        if (whichgun>3) 
          whichgun=3;
        shotsrc[0]=gPlayerPos[0]+guns[whichgun][0]*PLAYER_WIDTH/2;
        shotsrc[1]=gPlayerPos[1]+gNear;
        shotsrc[2]=gPlayerPos[2]+guns[whichgun][1]*PLAYER_HEIGHT/2;
        shotdiff[0] = -guns[whichgun][1]*PLAYER_HEIGHT/20;
        shotdiff[1] =  guns[whichgun][0]*PLAYER_WIDTH/20;

        /* Maybe do additive transparency? */
        glColor3f(.4f, .6f, 1.f);
        glBegin(GL_TRIANGLES);
        glVertex3f(shotsrc[0]-shotdiff[0], shotsrc[1], shotsrc[2]-shotdiff[1]);
        glVertex3f(shotsrc[0]+shotdiff[0], shotsrc[1], shotsrc[2]+shotdiff[1]);
        glVertex3fv(gShotDest);
        glEnd();
      }

    /* Draw starfield? */

    /* Draw gun ports */
    for (i=NUM_TRENCH_SEGMENTS-1; i>=0; --i)
      if (gGunPorts[i])
        {
          int which;
          for (which=0; which<GUN_PORT_POSSIBLE; ++which)
            if (gGunPorts[i] & (1<<which))
              { /* Gun port present in position which */
                glPushMatrix();
                if (which<3)
                  {
                    glTranslatef(TRENCH_LEFT, TRENCH_DEPTH*i+GUN_PORT_Y, 
                                 TRENCH_BOTTOM+(which+.5)*(TRENCH_TOP-TRENCH_BOTTOM)/3);
                    glCallList(gModelDisplayList+Model_GunLeft);
                  }
                else if (which<6)
                  {
                    glTranslatef(TRENCH_LEFT+(which-2.5)*(TRENCH_RIGHT-TRENCH_LEFT)/3, 
                                 TRENCH_DEPTH*i+GUN_PORT_Y, TRENCH_BOTTOM);
                    glCallList(gModelDisplayList+Model_GunFloor);
                  }
                else
                  {
                    glTranslatef(TRENCH_RIGHT, TRENCH_DEPTH*i+GUN_PORT_Y, 
                                 TRENCH_BOTTOM+(which-5.5)*(TRENCH_TOP-TRENCH_BOTTOM)/3);
                    glCallList(gModelDisplayList+Model_GunRight);
                  }
                glPopMatrix();
              }
        }

    /* Draw missiles */
    for (i=0; i<MISSILES_MAX; ++i)
      if (gMissiles[i].Enabled)
        {
          glPushMatrix();
          glTranslatef(gMissiles[i].Pos[0], 
                       gMissiles[i].Pos[1], 
                       gMissiles[i].Pos[2]);
          glRotatef(gMissiles[i].Rotation, 
                    gMissiles[i].Axis[0], 
                    gMissiles[i].Axis[1], 
                    gMissiles[i].Axis[2]);
          glCallList(gModelDisplayList+Model_Missile);
          glPopMatrix();
        }

    /* ---- Draw transparent things back to front ---- */
    glEnable(GL_BLEND);

    /* Draw obstacles and explosions */
    for (i=NUM_TRENCH_SEGMENTS-1; i>=0; --i)
      {
        int whichexp;
        for (whichexp=0; whichexp<EXPLOSIONS_MAX; ++whichexp)
          if (gExplosions[whichexp].Countdown>0 && 
              ((int) (gExplosions[whichexp].Pos[1]/TRENCH_DEPTH))==i)
            { /* Draw a cool explosion */
              int j, p;
              float alpha = gExplosions[whichexp].Countdown/
                              EXPLOSIONS_DURATION;
              float delta = (1.f+alpha)*EXPLOSION_PARTICLE_SIZE*.25f;
              float center[3];
              for (j=0; j<3; ++j)
                center[j]=gExplosions[whichexp].Pos[j];
              glBegin(GL_QUADS);
              for (p=0; p<EXPLOSIONS_PARTICLES; ++p)
                {
                  float pos[3];
                  /* I'm relying on OpenGL to clip the color values */
                  glColor4f(1.0f, /* full red */
                            2.0f-2*gExplosions[whichexp].Particles[p].Color,
                                  /* start full green, but fade to 0 
                                     starting halfway through */
                            1.0f-2*gExplosions[whichexp].Particles[p].Color, 
                                  /* start full blue, but fade to 0 
                                     halfway through */
                            alpha);
                  for (j=0; j<3; ++j)
                    pos[j] = center[j] + 
                             gExplosions[whichexp].Particles[p].Vect[j]*
                                          (1.f-alpha)*EXPLOSIONS_RADIUS*2;
                  glVertex3f(pos[0]-delta, pos[1], pos[2]+delta);
                  glVertex3f(pos[0]-delta, pos[1], pos[2]-delta);
                  glVertex3f(pos[0]+delta, pos[1], pos[2]-delta);
                  glVertex3f(pos[0]+delta, pos[1], pos[2]+delta);
                }
              glEnd();
            }
        if (gObstacles[i])
          {
            int block;
            float scalef=1.f/(1.0+0.3*(i+1.000001-gPlayerPos[1]));
            glColor4f(scalef*.5, scalef, 0.f, 0.7f);
          
            switch (gObstacles[i] & OBSTACLES_TYPE_MASK)
              {
                case OBSTACLES_HORIZ:
                  for (block=0; block<OBSTACLES_ROWS; ++block)
                    if (gObstacles[i] & (1<<block))
                      {
                        float z1=block*OBSTACLE_HIGH;
                        float z2=z1+OBSTACLE_HIGH;
                        glBegin(GL_QUADS);
                        glVertex3f(TRENCH_LEFT, TRENCH_DEPTH*i+OBSTACLE_Y, z1);
                        glVertex3f(TRENCH_RIGHT, TRENCH_DEPTH*i+OBSTACLE_Y, z1);
                        glVertex3f(TRENCH_RIGHT, TRENCH_DEPTH*i+OBSTACLE_Y, z2);
                        glVertex3f(TRENCH_LEFT, TRENCH_DEPTH*i+OBSTACLE_Y, z2);
                        glEnd();
                      }
                  break;
                case OBSTACLES_VERT:
                  for (block=0; block<OBSTACLES_COLUMNS; ++block)
                    if (gObstacles[i] & (1<<block))
                      {
                        float x1=block*OBSTACLE_WIDE;
                        float x2=x1+OBSTACLE_WIDE;
                        glBegin(GL_QUADS);
                        glVertex3f(x1, TRENCH_DEPTH*i+OBSTACLE_Y, TRENCH_BOTTOM);
                        glVertex3f(x2, TRENCH_DEPTH*i+OBSTACLE_Y, TRENCH_BOTTOM);
                        glVertex3f(x2, TRENCH_DEPTH*i+OBSTACLE_Y, TRENCH_TOP);
                        glVertex3f(x1, TRENCH_DEPTH*i+OBSTACLE_Y, TRENCH_TOP);
                        glEnd();
                      }
                  break;
                case OBSTACLES_WALL:
                  for (block=0; block<OBSTACLES_GRID; ++block)
                    if (gObstacles[i] & (1<<block))
                      {
                        int whichx = block % OBSTACLES_COLUMNS;
                        int whichz = block / OBSTACLES_COLUMNS;
                        float x1=whichx*OBSTACLE_WIDE;
                        float x2=x1+OBSTACLE_WIDE;
                        float z1=whichz*OBSTACLE_HIGH;
                        float z2=z1+OBSTACLE_HIGH;
                        glBegin(GL_QUADS);
                        glVertex3f(x1, TRENCH_DEPTH*i+OBSTACLE_Y, z1);
                        glVertex3f(x2, TRENCH_DEPTH*i+OBSTACLE_Y, z1);
                        glVertex3f(x2, TRENCH_DEPTH*i+OBSTACLE_Y, z2);
                        glVertex3f(x1, TRENCH_DEPTH*i+OBSTACLE_Y, z2);
                        glEnd();
                      }
                  break;
                default:
                  assert(0); /* should never get here */
              }
          }
      }

    if (! gDemoMode)
      { /* This stuff is distracting in demo mode */

        /* ---- Draw "always in front" things last ---- */
        /* (disable z writes or compares since the remaining are "on top") */
        glDisable(GL_DEPTH_TEST);
        glLoadIdentity();

        gluLookAt(0.f, 0.f, 0.f,  /* camera position */
                  0.f, 1.f, 0.f,  /* look at vector */
                  0.f, 0.f, 1.f); /* up vector */

        if (gShieldCountdown)
          { /* Draw shield loss effect */
            float effectamount = sin(gShieldCountdown*gShieldCountdown*16);
            float ztop=tan(DegreesToRadians(gFov/2));
            float zbot=-ztop;
            float xright=ztop*gWidth/gHeight;
            float xleft=-xright;
            glColor4f(1.0, /* Very red */
                      0.3, /* A little green */
                      0.0, /* No blue */
                      effectamount); /* Fade amount */
            glBegin(GL_QUADS);
            glVertex3f(xleft,  1.f, ztop);
            glVertex3f(xleft,  1.f, zbot);
            glVertex3f(xright, 1.f, zbot);
            glVertex3f(xright, 1.f, ztop);
            glEnd();
          }

        { /* Draw stats */
          float z=tan(DegreesToRadians(gFov/2));
          float x=z*0.97*gWidth/gHeight;
          z*=0.85;

          glLineWidth(gHeight/200);
          glColor4f(1, 1, 1, /* White */
                    1);      /* Opaque */
      
          sprintf(gMessageBuffers[1], "Score: %d", gScore);
          CornerText(gMessageBuffers[1], SCORE_SCALE, false, x, z);
          sprintf(gMessageBuffers[1], "Shields: %d", gShields);
          CornerText(gMessageBuffers[1], SCORE_SCALE, true, x, z);
        }

        /* Draw message */
        if (gMessage)
          {
            glLineWidth(gHeight/100);
            glColor4f(0.5*gMessageCountdown-0.25, /* Not very red */
                      0.9*gMessageCountdown-0.25, /* More green */
                      gMessageCountdown,          /* Mostly blue */
                      gMessageCountdown);         /* Fading out */

            CenterText(gMessage, MESSAGE_SCALE, \
                              1.25-.25*gMessageCountdown, .2);
          }
        /* Draw cursor? */

        /* Don't do these in demo mode either.
           In demo mode there is still text to draw.
         */
        glutSwapBuffers();

        glEnable(GL_DEPTH_TEST);
      }

    CheckForGLError("DrawWorld");
  }