/* (c) Copyright 2001 Josh Levenberg */

#include <stdio.h>
#include <stdlib.h>
#include <GL/glut.h>
#include "game.h"
#include "physics.h"
#include "timems.h"
#include "draw.h"

static void GameInit(void);
static void GameClose(void);
static void GameDraw(void);
static void GameIdle(void);
static void GameMouseClick(int button, int state, int x, int y);
static void GameMouseMove(int x, int y);

GlutState gGameMode =
  { 
    GameInit,
    GameClose,
    GameDraw,
    GameIdle,
    GameMouseClick,
    GameMouseMove
  };

static double sTime;
static double sTimeInitial;
static int sEpochs;
static unsigned sNumFrames;

#define MAX_UNSIGNED (((double)((unsigned)-1))+1)

/* Switch into game mode.  Note that we may be switching from
   the intro (which will initialize the physics) or from paused
   (minimized) and we should simply resume from whatever state
   we were in.
 */
static void GameInit()
  {
    sTimeInitial = GetTimeInMS()/1000.0;
    sTime = sTimeInitial;
    sNumFrames = 0;
    sEpochs = 0;
  }

static void GameClose()
  {
    double time_elapsed = sTime-sTimeInitial+(MAX_UNSIGNED/1000.)*sEpochs;
    if (time_elapsed>0)
      {
        double framerate = sNumFrames/time_elapsed;
        printf("%g frames per second\n", framerate);
      }
  }

static void GameDraw()
  {
    DrawWorld();
    ++sNumFrames;
  }

static void GameIdle()
  {
    double time = GetTimeInMS()/1000.0;
    double delta_time = time-sTime;
    /* deal with overflow */
    if (delta_time<0)
      {
        delta_time += MAX_UNSIGNED/1000.0;
        ++sEpochs;
      }
    sTime = time;

    if (delta_time>1)
      { /* if less than 1 frame per second, 
           only advance frame by one second 
           (this comes up when debugging)
        */
        delta_time=1;
      }
    PhysicsAdvance(delta_time);
    
    glutPostRedisplay();
  }

static void GameMouseClick(int button, int state, int x, int y)
  {
    if (state==GLUT_DOWN)
      PhysicsFire();
  }

static void GameMouseMove(int x, int y)
  {
    float aimx=((float)x)/gWidth;
    float aimz=1.0f - ((float)y)/gHeight;
    if (aimx<0)
      aimx=0;
    if (aimx>1)
      aimx=1;
    if (aimz<0)
      aimz=0;
    if (aimz>1)
      aimz=1;
    PhysicsAim(aimx, aimz);
  }
