/* (c) Copyright 2001 Josh Levenberg */

#include <GL/glut.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "gameover.h"
#include "intro.h"
#include "timems.h"
#include "world.h"
#include "text.h"

/* Size of font */
#define MESSAGE_SCALE (1.f/700.f)

static void GameOverInit(void);
static void GameOverClose(void);
static void GameOverDraw(void);
static void GameOverIdle(void);
static void GameOverMouseClick(int button, int state, int x, int y);
static void GameOverMouseMove(int x, int y);

GlutState gGameOverMode =
  { 
    GameOverInit,
    GameOverClose,
    GameOverDraw,
    GameOverIdle,
    GameOverMouseClick,
    GameOverMouseMove
  };

static double sTime;
static double sTimeElapsed;

static void GameOverInit()
  {
    sTime = GetTimeInMS()/1000.0;
    sTimeElapsed = 0;
    printf("Game over! Level: %d  Score: %d\n", gLevel, gScore);
  }

static void GameOverClose()
  {
  }

/* Draw one of three messages depending on how much time 
   has elapsed.
 */
static void GameOverDraw()
  {
    float percentthrough;
    int whichmsg;

    glClear(GL_COLOR_BUFFER_BIT);

    glLoadIdentity();

    gluLookAt(0.f, 0.f, 0.f,  /* camera position */
              0.f, 1.f, 0.f,  /* look at vector */
              0.f, 0.f, 1.f); /* up vector */

    glDisable(GL_DEPTH_TEST);

    if (sTimeElapsed<5.f)
      {
        strcpy(gMessageBuffers[0], "Game Over");
        strcpy(gMessageBuffers[1], "");
        percentthrough = sTimeElapsed/5.f;
      }
    else if (sTimeElapsed<10.f)
      {
        sprintf(gMessageBuffers[0], "Level: %d", gLevel);
        sprintf(gMessageBuffers[1], "Score: %d", gScore);
        percentthrough = (sTimeElapsed-5.f)/5.f;
      }
    else
      {
        strcpy(gMessageBuffers[0], "Q or Esc");
        strcpy(gMessageBuffers[1], "to Quit");
        percentthrough = (sTimeElapsed-10.f)/5.f;
      }

    for (whichmsg=0; whichmsg<2; ++whichmsg)
      {
        glLineWidth(gHeight/100);
        glColor4f(1.0f,   /* Very red */
                  0.75f,  /* Lots of green */
                  0.4f,   /* Not much blue */
                  1.f-percentthrough); /* Fading out */

        CenterText(gMessageBuffers[whichmsg], MESSAGE_SCALE, 
                1.f+.25f*percentthrough, .1f-.2f*whichmsg);
      }
    
    glutSwapBuffers();

    glEnable(GL_DEPTH_TEST);
  }

/* Keep track of time so messages fade away
   and so we know when to switch to intro mode
 */
static void GameOverIdle()
  {
    double time = GetTimeInMS()/1000.0;
    /* Ignore if we are overflowing.
       Not quite as accurate as the code in game.c, but simpler
     */
    if (time>=sTime)
      sTimeElapsed += time-sTime;
    sTime = time;

    if (sTimeElapsed>15.0)
      SwitchToGlutState(&gIntroMode);
    else
      glutPostRedisplay();
  }

static void GameOverMouseClick(int button, int state, int x, int y)
  {
    /* Restart if player clicks, but have a grace period so
       the player has a chance to notice the game is over */
    if (state==GLUT_DOWN && sTimeElapsed>1.5f)
      SwitchToGlutState(&gIntroMode);
  }

static void GameOverMouseMove(int x, int y)
  {
  }
