/* (c) Copyright 2001 Josh Levenberg */

#include <GL/glut.h>
#include <stdlib.h>
#include <time.h>
#include "init.h"
#include "model.h"
#include "intro.h"


/* Setup OpenGL state we are mostly going to leave alone. */
static void InitializeGLDefaultState()
  {
    glClearColor(0.0f, 0.0f, 0.0f, 0.5f);   /* Black background */

    /* Depth buffer setup */
    glClearDepth(1.0f);
    glEnable(GL_DEPTH_TEST);                /* Enables depth testing */
    glDepthFunc(GL_LEQUAL);                 /* Depth comparison test */

    /* Enable back face culling */
    /* (actually, the models will be built with mostly 
        front facing polygons, so this may not be a performance win */
    glEnable(GL_CULL_FACE);
    glCullFace(GL_BACK);
    glPolygonMode(GL_FRONT, GL_FILL);
    glFrontFace(GL_CCW);                    /* Counter clockwise 
                                               vertex order */

    /* Disable lighting and shading */
    glDisable(GL_LIGHTING);
    glDisable(GL_COLOR_MATERIAL);
    glShadeModel(GL_FLAT);

    /* Disable advanced features for speed */
    glDisable(GL_POLYGON_SMOOTH);
    glDisable(GL_POLYGON_STIPPLE);
    glDisable(GL_FOG);

    /* Want cool transparency effects */
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

    CheckForGLError("InitializeGLDefaultState");
  }

/* atexit function to delete display lists created in
   CompileModelsToDisplayLists() below
 */
static void DeleteDisplayLists(void)
  {
    glDeleteLists(gModelDisplayList, NumModels);
  }

static void CompileModelsToDisplayLists()
  {
    int which;
    gModelDisplayList=glGenLists(NumModels);

    /* Vertex arrays are used by model functions */
    glEnableClientState(GL_VERTEX_ARRAY);

    for (which=0; which<NumModels; ++which)
      {
        glNewList(gModelDisplayList+which, GL_COMPILE);
        gDrawModelFuncArray[which]();
        glEndList();
      }

    /* Vertex arrays are ONLY used by model functions */
    glDisableClientState(GL_VERTEX_ARRAY);

    CheckForGLError("CompileModelsToDisplayLists");

    atexit(DeleteDisplayLists);
  }

void Initialize()
  {
    InitializeGLDefaultState();

    CompileModelsToDisplayLists();

#ifdef NDEBUG
    srand((unsigned)time(NULL));
#else
    srand(0);
#endif

    SwitchToGlutState(& gIntroMode);
  }
