/* (c) Copyright 2001 Josh Levenberg */

#include <gl/glut.h>
#include "intro.h"
#include "game.h"
#include "physics.h"
#include "timems.h"
#include "text.h"
#include "glswmath.h"
#include "draw.h"
#include "world.h"

static void IntroInit(void);
static void IntroClose(void);
static void IntroDraw(void);
static void IntroIdle(void);
static void IntroMouseClick(int button, int state, int x, int y);
static void IntroMouseMove(int x, int y);

GlutState gIntroMode =
  { 
    IntroInit,
    IntroClose,
    IntroDraw,
    IntroIdle,
    IntroMouseClick,
    IntroMouseMove
  };

static const char *sMessages[] =
  { "GL SW",
    "by Josh Levenberg",
    "Click to begin",
    "F1 - Full Screen",
    "P - Minimize",
    "Esc or Q - Quit",
    "Guns - 15 pts",
    "Missiles - 10 pts"
  };

#define MESSAGE_DURATION (5.f)
#define MESSAGE_INTERVAL (1.2f)
#define MESSAGE_SCALE (1.f/700.f)

enum { NumMessages = sizeof(sMessages)/sizeof(sMessages[0]) };

static int sFirstMessage;
static double sTime;
static double sTextProgress;

/* Whether the demo has started.
   Set true after the messages have been shown once.
 */
static bool sDemoActive;

static void IntroInit()
  {
    sTime = GetTimeInMS()/1000.0;
    sTextProgress = 0;
    sFirstMessage = 0;
    sDemoActive = false;
  }

static void IntroClose()
  {
  }

static void IntroDraw()
  {
    float percentthrough = (float) (sTextProgress/MESSAGE_DURATION);
    int msg;

    if (sDemoActive)
      /* draw a demo world */
      DrawWorld();
    else
      glClear(GL_COLOR_BUFFER_BIT);

    glLoadIdentity();

    gluLookAt(0.f, 0.f, 0.f,  /* camera position */
              0.f, 1.f, 0.f,  /* look at vector */
              0.f, 0.f, 1.f); /* up vector */

    glDisable(GL_DEPTH_TEST);
    
    for (msg=sFirstMessage; msg<NumMessages; ++msg)
      {
        float depth = 1.f+2.f*percentthrough;
        glLineWidth(gHeight/(100.f*depth));

        glColor4f(1.0f, 0.75f, 0.4f,   /* Orangish yellow */
                  1.f-percentthrough); /* Fading out */

        CenterText(sMessages[msg], MESSAGE_SCALE, depth, 
                    1.3f*percentthrough-.55f);

        percentthrough -= MESSAGE_INTERVAL/MESSAGE_DURATION;
        if (percentthrough < 0.f)
          break;
      }

    glutSwapBuffers();

    glEnable(GL_DEPTH_TEST);
  }

static void IntroIdle()
  {
    double time = GetTimeInMS()/1000.0;
    /* Ignore if we are overflowing.
       Not quite as accurate as the code in game.c, but simpler
     */
    if (time>=sTime)
      {
        sTextProgress += time-sTime;
        if (sDemoActive)
          PhysicsAdvance(time-sTime);
      }
    sTime = time;

    if (sTextProgress>MESSAGE_DURATION)
      {
        ++sFirstMessage;
        if (sFirstMessage>=NumMessages)
          { /* finished displaying messages */
            /* start over */
            sFirstMessage=0;
            sTextProgress=0.f;
            if (! sDemoActive)
              { /* start demo */
                sDemoActive=true;
                gDemoMode = true;
                PhysicsInit(Random(9)+1);
              }
          }
        else
          {
            sTextProgress -= MESSAGE_INTERVAL;
          }
      }
    glutPostRedisplay();
  }

static void IntroMouseClick(int button, int state, int x, int y)
  {
    if (state==GLUT_DOWN)
      { /* Begin the game */
        gDemoMode = false;
        PhysicsInit(1);
        SwitchToGlutState(& gGameMode);
      }
  }

static void IntroMouseMove(int x, int y)
  {
  }
