/* (c) Copyright 2001 Josh Levenberg */

/* Instances of global variables defined in world.h */
/* Also functions for creating explosions, missiles, 
   and initializing gun port position data. */

#include <stdlib.h>
#include <math.h>
#include <assert.h>
#include "glswmath.h"
#include "world.h"

bool gDemoMode;

float gPlayerPos[3];
float gPlayerAim[2];

int gLevel;
unsigned gScore;
int gHowFarThroughLevel;
float gSpeedUp;

float gShootCountdown;
float gCoolDown;
bool gFireWhenCoolDown;
float gShootAim[2];
float gShotDest[3];

int gShields;
float gShieldCountdown;

unsigned long gObstacles[NUM_TRENCH_SEGMENTS];

unsigned short gGunPorts[NUM_TRENCH_SEGMENTS];
float gGunPortXZ[GUN_PORT_POSSIBLE][2];

MissileInfo gMissiles[MISSILES_MAX];

ExplosionInfo gExplosions[EXPLOSIONS_MAX];

const char *gMessage;
float gMessageCountdown;

/* Create an explosion with a particle system */
/* If all the explosion slots are full, replaces the one
   closest to finishing.
 */
void CreateExplosion(float location[3], ExplodeDirection dir)
  {
    int i, whichmincountdown=0;
    float mincountdown=gExplosions[0].Countdown;
    for (i=1; i<EXPLOSIONS_MAX; ++i)
      if (gExplosions[i].Countdown < mincountdown)
        {
          mincountdown=gExplosions[i].Countdown;
          whichmincountdown=i;
        }
    for (i=0; i<3; ++i)
      gExplosions[whichmincountdown].Pos[i]=location[i];
    gExplosions[whichmincountdown].Countdown = EXPLOSIONS_DURATION;
    for (i=0; i<EXPLOSIONS_PARTICLES; ++i)
      {
        int j;
        gExplosions[whichmincountdown].Particles[i].Color=
            FRand()*.7f;  /* random number 0 .. .7 */
        for (j=0; j<3; ++j)
          gExplosions[whichmincountdown].Particles[i].Vect[j] =
              FRand()*2.f - 1.f; /* random number -1 .. 1 */
      }

    /* if we are against a wall, flip the vectors pointing
       into the wall so that we can see all of the particles */
    switch (dir)
      {
        case EXPLODE_ALL:
          /* already exploding in all directions */
          break;
        case EXPLODE_RIGHT:
          for (i=0; i<EXPLOSIONS_PARTICLES; ++i)
            gExplosions[whichmincountdown].Particles[i].Vect[0] = (float)
              fabs(gExplosions[whichmincountdown].Particles[i].Vect[0]);
          break;
        case EXPLODE_UP:
          for (i=0; i<EXPLOSIONS_PARTICLES; ++i)
            gExplosions[whichmincountdown].Particles[i].Vect[2] = (float)
              fabs(gExplosions[whichmincountdown].Particles[i].Vect[2]);
          break;
        case EXPLODE_LEFT:
          for (i=0; i<EXPLOSIONS_PARTICLES; ++i)
            gExplosions[whichmincountdown].Particles[i].Vect[0] = (float)
              -fabs(gExplosions[whichmincountdown].Particles[i].Vect[0]);
          break;
        default:
          assert(0); /* should never get here */
      }
  }


/* fill gGunPortXZ array */
void InitGunPortXZArray(void)
  {
    int i;
    for (i=0; i<GUN_PORT_PER_WALL; ++i)
      {
        gGunPortXZ[i]                    [0]=TRENCH_LEFT;
        gGunPortXZ[i]                    [1]=TRENCH_BOTTOM+
                                      (i+.5f)*(TRENCH_TOP-TRENCH_BOTTOM)/3;
        gGunPortXZ[i+  GUN_PORT_PER_WALL][0]=TRENCH_LEFT+
                                      (i+.5f)*(TRENCH_RIGHT-TRENCH_LEFT)/3;
        gGunPortXZ[i+  GUN_PORT_PER_WALL][1]=TRENCH_BOTTOM;
        gGunPortXZ[i+2*GUN_PORT_PER_WALL][0]=TRENCH_RIGHT;
        gGunPortXZ[i+2*GUN_PORT_PER_WALL][1]=TRENCH_BOTTOM+
                                      (i+.5f)*(TRENCH_TOP-TRENCH_BOTTOM)/3;
      }
  }

/* Create a missile assuming there is a slot open */
void CreateMissile(float pos[3], float vel[3])
  {
    int missilenum;
    for (missilenum=0; missilenum<MISSILES_MAX; ++missilenum)
      if (! gMissiles[missilenum].Enabled)
        break;
    if (missilenum<MISSILES_MAX)
      {
        int i;
        gMissiles[missilenum].Enabled=true;
        for (i=0; i<3; ++i)
          {
            gMissiles[missilenum].Pos[i]=pos[i];
            gMissiles[missilenum].Velocity[i]=vel[i];
            /* random axis of rotation */
            gMissiles[missilenum].Axis[i]=2.f*FRand()-1.f;
          }
        /* start at a random point in the rotation cycle */
        gMissiles[missilenum].Rotation=360.f*FRand();
      }
  }
